##------------------------------------------------------------##
##               Generalized Linear Models                    ##
##                       John Fox                             ##
##                      York SPIDA                            ##
##                         2012                               ##
##------------------------------------------------------------##

# Chilean plebiscite data
library(car)

?Chile

## LOGIT AND PROBIT MODELS FOR DICHOTOMOUS DATA - Section 4, Slide 49 ##
Chile$yes <- with(Chile, ifelse(vote == "Y", 1, ifelse(vote=="N", 0, NA)))
xtabs(~ vote + yes, data=Chile)

Chile <- na.omit(Chile[, c("yes", "statusquo")])

plot(jitter(yes, factor=1/4) ~ statusquo, data=Chile, xlab="Support for the Status Quo", ylab="Voting Intention")
abline(h=c(0,1), lty=2)

# Linear-Probability Model - Section 4.1, Slide 54
mod.lin <- lm(yes ~ statusquo, data=Chile)
summary(mod.lin)

abline(mod.lin, lwd=2) # linear probability model by OLS

# The Constrained Linear-Probability Model - Section 4.1, Slide 58
mod <- glm(yes ~ statusquo, family=binomial, data=Chile) # logit model by ML
summary(mod)

sq <- with(Chile, seq(min(statusquo), max(statusquo), length=1000))
fit <- predict(mod, newdata=data.frame(statusquo=sq), type="response")
lines(sq, fit, lwd=2)

mod.lo <- loess(yes ~ statusquo, span=0.5, degree=0, data=Chile) # local average smoother, mentioned on slide 53
fit.lo <- predict(mod.lo, newdata=data.frame(statusquo=sq))
lines(sq, fit.lo, lwd=2, lty=2)

# Logit Model for Multiple Regression - Section 4.3, Slide 70
# logistic regression for SLID data
SLID <- read.table(
"http://socserv.socsci.mcmaster.ca/jfox/Books/Applied-Regression-2E/datasets/SLID-women.txt", 
    header=TRUE)
    
some(SLID)
summary(SLID) 

SLID$region <- factor(SLID$region,       # re-order levels
    levels=c("Atlantic", "Quebec", "Ontario", "Prairies", "BC"))

mod.slid.1 <- glm(working ~ region + kids0004 + kids0509 + kids1014 
    + familyIncome + education, family=binomial, data=SLID)
# Statistical Inference: Likelihood Ratio Tests - Section 4.3, Slide 72
Anova(mod.slid.1)

mod.slid.2 <- glm(working ~ region + kids0004 + kids0509 + familyIncome 
    + education, family=binomial, data=SLID)
summary(mod.slid.2)

confint(mod.slid.2) # LR-based intervals, preferable

confint.default(mod.slid.2) # Wald-based intervals

# Probit Model for Multiple Regression - Section 4.3, Slide 70    
# alternative probit model
mod.slid.probit <- glm(working ~ region + kids0004 + kids0509 + familyIncome 
    + education, family=binomial(link=probit), data=SLID)
# Expressing the model in terms of odds - Section 4.3, Slide 71
coef(mod.slid.probit)*pi/sqrt(3)
coef(mod.slid.2)

plot(fitted(mod.slid.2), fitted(mod.slid.probit))
abline(0, 1)

logLik(mod.slid.2)
logLik(mod.slid.probit)


# Effect plots for the logit model
library(effects)

plot(allEffects(mod.slid.2), ask=FALSE)

with(SLID, quantile(education,  probs=c(.1, .9)))
with(SLID, quantile(familyIncome,  probs=c(.1, .9)))
eff <- allEffects(mod.slid.2, xlevels=list(education=10:17, 
    familyIncome=seq(11.6, 44.3, length=10)))
plot(eff, ask=FALSE) 

# Poisson regression for Ornstein's interlocking-directorate data
# Poisson GLMS for Count Data - Section 6.0, Slide 90
?Ornstein

(tab <- table(Ornstein$interlocks))

(x <- as.numeric(names(tab)))

plot(x, tab, type='h', xlab='Number of Interlocks', ylab='Frequency')
points(x, tab, pch=16)

mod.ornstein <- glm(interlocks ~ assets + nation + sector, 
                    family=poisson, data=Ornstein)
summary(mod.ornstein)

Anova(mod.ornstein)

# Quasi-Poisson model (Details in Section 9.0, Slide 111
mod.ornstein.qp <- glm(interlocks ~ assets + nation + sector, 
                       family=quasipoisson, data=Ornstein)
summary(mod.ornstein.qp)

Anova(mod.ornstein.qp)

Anova(mod.ornstein.qp, test="F")

# effect plot, based on quasi-Poisson model

plot(allEffects(mod.ornstein.qp), ask=FALSE)

## GLM DIAGNOSTICS - Section 7.1, Slide 94 ##

# unusual data, Section 7.1.3, Slide 98
influencePlot(mod.ornstein.qp, id.n=1)

influenceIndexPlot(mod.ornstein.qp, id.n=1)

D <- dfbeta(mod.ornstein.qp)
head(D)

plot(D[,"assets"])  # index plot
abline(h=c(-3.389e-06, 0, 3.389e-06), lty=2) #  +/- SE(B)
identify(D[,"assets"])

# Nonlinearity Diagnostics - Section 7.2, Slide 99
crPlot(mod.ornstein.qp, "assets", span=.9)

mod.ornstein.qp.1 <- glm(interlocks ~ log10(assets) + nation + sector, 
                         family=quasipoisson, data=Ornstein)
summary(mod.ornstein.qp.1)
deviance(mod.ornstein.qp)

crPlot(mod.ornstein.qp.1, "log10(assets)")

influencePlot(mod.ornstein.qp.1)

## LOGIT AND PROBIT MODELS FOR CONTINGENCY TABLES - Section 8, Slide 100 ##

# Binomial logit model for American Voter data, Section 8.1, Slide 100
closeness <- factor(rep(c('one.sided', 'close'), c(3, 3)), 
                    levels=c('one.sided', 'close'))   
preference <- factor(rep(c('weak', 'medium', 'strong'), 2), 
                     levels=c('weak', 'medium', 'strong'))
voted <- c(91, 121, 64, 214, 284, 201)
did.not.vote <- c(39, 49, 24, 87, 76, 25)

AmVoter <- data.frame(did.not.vote, voted, closeness, preference)
AmVoter

options("contrasts")
options(contrasts=c("contr.sum", "contr.poly")) # "deviation" regressors

amvoter <- glm(cbind(voted, did.not.vote) ~ closeness*preference, 
               family=binomial, data=AmVoter)  # "saturated" model
summary(amvoter)

amvoter0 <- update(amvoter, . ~ . - closeness:preference)
anova(amvoter0, amvoter, test="Chisq")  # LR test of interaction

Anova(amvoter)

Anova(amvoter, type=3)

# Loglinear model for American Voter data, Section 8.2, Slide 107
closeness <- rep(closeness, 2)
preference <- rep(preference, 2)
count <- c(voted, did.not.vote)
turnout <- factor(rep(c("voted", "did.not.vote"), each=6))
AmVoter <- data.frame(closeness, preference, turnout, count)
AmVoter

amvoter <- glm(count ~ closeness*preference*turnout, family=poisson,
               data=AmVoter)  # saturated model
summary(amvoter)

Anova(amvoter)

