##------------------------------------------------------------##
##                Basic Programming in R                      ##
##                       John Fox                             ##
##                      York SPIDA                            ##
##                         2012                               ##
##------------------------------------------------------------##


# Preliminary programming example

library(car) # for data

mod.duncan <- lm(prestige ~ income + education, data=Duncan)
cook <- sqrt(cooks.distance(mod.duncan))
plot(hatvalues(mod.duncan), rstudent(mod.duncan), cex=(10/max(cook))*cook)
abline(h=c(-2, 0, 2), lty=2)
abline(v=c(2, 3)*length(coef(mod.duncan))/length(rstudent(mod.duncan)), lty=1)
identify(hatvalues(mod.duncan), rstudent(mod.duncan), rownames(Duncan))

inflPlot <- function(model, scale=10, col=c(1, 2),
    identify=TRUE, labels=names(rstud), ... ) {
    # Plot hatvalues, Studentized residuals, and Cook's distances
    #  for a linear or generalized linear model
    # Arguments:
    #  model: an lm or glm model object
    #  scale: a scaling factor for the circles representing Cook's D
    #  col: colors for non-noteworthy and noteworthy points
    #  identify points: label noteworthy points (TRUE or FALSE)
    #  labels: for identified points
    hatval <- hatvalues(model)
    rstud <- rstudent(model)
    cook <- sqrt(cooks.distance(model))
    scale <- scale/max(cook, na.rm = TRUE)
    p <- length(coef(model))
    n <- length(rstud)
    cutoff <- sqrt(4/(n - p))
    plot(hatval, rstud, xlab = "Hat-Values", ylab = "Studentized Residuals",
        cex=scale*cook, col=ifelse(cook > cutoff, col[2], col[1]), ...)
    abline(v = c(2, 3)*p/n, lty = "dashed")
    bonf <- qt(.025/n, df = n - p - 1, lower.tail=FALSE)
    abline(h=c(-bonf, -2, 0, 2, bonf), lty="dashed")
    if (identify){
        noteworthy <- cook > cutoff | abs(rstud) > bonf | hatval > 2*p/n
        pos <- ifelse(hatval - mean(range(hatval, na.rm=TRUE)) <= 0, 4, 2)
        text(hatval[noteworthy], rstud[noteworthy], labels[noteworthy],
            pos = pos[noteworthy])
        return(which(noteworthy))
    }
    else return(invisible(NULL))
}

inflPlot(mod.duncan)

inflPlot(mod.duncan, ylim=c(-3, 4), las=1, col=gray(c(0.5, 0)))

# Basic matrix operations

(A <- matrix(c(1, 2, -4, 3, 5, 0), nrow=2, ncol=3))
(B <- matrix(1:6, 2, 3))
(C <- matrix(c(2, -2, 0, 1, -1, 1, 4 ,4, -4), 3, 3, byrow=TRUE))

A + B  # addition
A - B  # subtraction
2*A    # product of a scalar and a matrix
-A     # negation

A %*% C  # matrix product

A %*% B  # not conformable for matrix multiplication

    # vectors are treated as row or column vectors as needed
    
a <- rep(1, 3)
b <- c(1, 5, 3)
C %*% a

a %*% C

a %*% b  # inner product

outer(a, b)  # outer product

t(B)  # matrix transpose

solve(C)  # matrix inverse
solve(C) %*% C  # check

    # illustration: naive computation of LS regression
    
X <- cbind(1, as.matrix(Prestige[,1:3]))  # attach the constant
y <- Prestige[, 4]
head(X)  # first 6 rows
head(y)

head(Prestige[, 4, drop=FALSE]) # as a one-column matrix

solve(t(X) %*% X) %*% t(X) %*% y  # LS coefficients

lm(prestige ~ education + income + women, data=Prestige) # check?

# Control structures

    # conditionals
    
        # if ... else
        
abs1 <- function(x) if (x < 0) -x else x
abs1(-5)
abs1(5)

abs1(-3:3)  # wrong! the first element, -3, controls the result

        # ifelse (vectorized)
        
abs2 <- function(x) ifelse(x < 0, -x, x)
abs2(-3:3)

        # cascading conditionals
        
sign1 <- function(x){
    if (x < 0) -1
        else if (x > 0) 1
            else 0
}

sign1(-5)

sign2 <- function(x){
    ifelse (x < 0, -1,
        ifelse(x > 0, 1, 0))
}

sign2(c(-5, 0, 10))

    # iteration
    
        # for loops
        
prod(1:5)    # ok for small numbers
gamma(5 + 1) # better
factorial(5) # best
factorial
        
fact1 <- function(x){
    if (x <= 1) return(1)
    f <- 1  # initialize
    for (i in 1:x) f <- f * i  # accumulate product
    f  # return result
}

fact1(5)

fact1(-5)  # wrong!
        
        # while loops
        
fact2 <- function(x){
    if ((!is.numeric(x)) || (x != floor(x))
         || (x < 0) || (length(x) > 1))
        stop("x must be a non-negative integer")
    i <- f <- 1  # initialize
    while (i <= x) {
        f <- f * i  # accumulate product
        i <- i + 1  # increment counter
        }
    f  # return result
}

fact2(5)
fact2(-5)

    # recursion
    
fact3 <- function(x){
    if (x <= 1) 1  # termination condition
    else x * fact3(x - 1)  # recursive call
}

fact3(3)

trace(fact3)
fact3(3)
untrace(fact3)

# Avoiding loops

    # the "apply" family
        
        # apply (arrays)
    
head(DavisThin, 10)  # first 10 rows
dim(DavisThin)

DavisThin$thin.drive <- apply(DavisThin, 1, sum) # row sums
head(DavisThin$thin.drive, 10)

apply(DavisThin, 2, mean) # column means

colMeans(DavisThin) # more efficient (rowMeans, colSums, rowSums too)

DavisThin$thin.drive <- NULL  # remove thin.drive
DavisThin[1, 2] <- DavisThin[2, 4] <- DavisThin[10, 3] <- NA  # some missing data
head(DavisThin, 10)

head(apply(DavisThin, 1, sum), 10)

head(apply(DavisThin, 1, function(x) 7*mean(x, na.rm=TRUE)), 10)


        # lapply, sapply (lists)

thin.list <- as.list(DavisThin)
str(thin.list)  # structure of the result

lapply(thin.list, mean, na.rm=TRUE) # returns list (na.rm passed to mean)

lapply(thin.list, function(x) mean(x, na.rm=TRUE)) # equivalent

sapply(thin.list, mean, na.rm=TRUE) # simplifies result

        # tapply (tables of statistics)
        
some(Moore)  # randomly sample 10 observations

with(Moore, tapply(conformity,
   list(Status=partner.status, Authoritarianism=fcategory), mean))
   
Moore$fcategory <- factor(Moore$fcategory, 
    levels=c("low", "medium", "high"))
Moore$partner.status <- factor(Moore$partner.status,
    levels=c("low", "high"))
    
with(Moore, tapply(conformity,
   list(Status=partner.status, Authoritarianism=fcategory), mean))

   
